CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C      READ INPUT PARAMETERS
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine readin(n_lev)

       common
     &/ran_var/iso
     &/pi/pi,two_pi
     &/fund_const/q,h,kb,am0,eps_0 
     &/dri/qh
     &/temp/tem,Vt
     &/mass/amd,amc,smh,hhm,tm(3),hm(3)
     &/nonp/af,af2,af4
     &/scatt_par/emax,de,w(10,5),tau_max(5),max_scatt_mech(5)     
     &/density/density,sound_velocity
     &/dielec_func/eps_high
     &/time_1/dt,dtau,tot_time
     &/force/fx,fy,fz
     &/select_acouctic/acoustic_scattering
     &/select_Coulomb/Coulomb_scattering
     &/select_intervalley_1/intervalley_zero_g
     &/select_intervalley_2/intervalley_zero_f
     &/select_intervalley_3/intervalley_first_g
     &/select_intervalley_4/intervalley_first_f
     &/sigma_acoustic/sigma_acoustic
     &/doping_regions/n_region
     &/coulomb/doping_density(5),Energy_debye(5)
     &/Def_pot_1/DefPot_zero_g
     &/Def_pot_2/DefPot_zero_f
     &/Def_pot_3/DefPot_first_g
     &/Def_pot_4/DefPot_first_f
     &/interval_phonons_1/phonon_zero_g
     &/interval_phonons_2/phonon_zero_f
     &/interval_phonons_3/phonon_first_g
     &/interval_phonons_4/phonon_first_f
   
       real kb
       real fx,fy,fz
       real nonparabolicity_factor

C      Define fundamental constants and general parameters

       iso=1345
       am0=9.11e-31
       h=1.05459e-34
       q=1.60219e-19
       qh=q/h  
       eps_0=8.85419e-12
       kb=1.38066e-23
       pi=4.*atan(1.0)
       two_pi = 2.*pi

C      Define time step and maximum simulation time

       dt=1e-14
       tot_time = 2.e-12

C      Set temperature and doping density in various regions

       tem=300.
       Vt=kb*tem/q
       n_region = 1
       doping_density(1) = 1.e24

C      Set the electric field

       fx = 0.       
       fy = 0
       fz = 20.E5

C      Define Longitudinal and transverse masses
 
       am_l = 0.91
       am_t = 0.19

C      Define non-parabolicity factors

       nonparabolicity_factor = 0.5 
               
C      Define high-frequency dielectric constant

       eps_high = 10.92
       eps_high = eps_high*eps_0

C      Define crystal density and sound velocity

       density = 2329.
       sound_velocity = 9040.

C      Define parameters for the scattering table

       emax=1.0
       de=emax/float(n_lev)

C      Select scattering mechanisms

c      Coulomb_scattering = 1
       acoustic_scattering = 1
       intervalley_zero_g = 1
       intervalley_zero_f = 1
       intervalley_first_g = 1
       intervalley_first_f = 1

C      Define coupling constants

       sigma_acoustic = 9.  !  [eV]
       DefPot_zero_g = 5.23E10   ! [eV/m]
       DefPot_zero_f = 5.23E10   ! [eV/m]
       DefPot_first_g = 4.   ! [eV]
       DefPot_first_f = 2.5   ! [eV]
   
       phonon_zero_g = 0.06  !  [eV]
       phonon_zero_f = 0.06  !  [eV]
       phonon_first_g = 0.018  !  [eV]
       phonon_first_f = 0.023  !  [eV]

C      Map parameters into internal variables used in the code

       amd = (am_l*am_t*am_t)**(1./3.)
       amc = 3./(1./am_l + 2./am_t)
       tm(1) = sqrt(amc/am_l)
       tm(2) = sqrt(amc/am_t)
       tm(3) = sqrt(amc/am_t)
       amd = amd*am0
       amc = amc*am0
       hm(1) = h/amc*tm(1)
       hm(2) = h/amc*tm(2)
       hm(3) = h/amc*tm(3)

       af = nonparabolicity_factor
       af2 = 2.*af
       af4 = 4.*af
       smh = sqrt(2.*amc)*sqrt(q)/h
       hhm =h/amc/q*h/2.

       return
       end  
       
