CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C       SELECT SCATTERING MECHANISM AND PERFORM 
C       THE SCATTERING PART THAT MODIFIES PARTICLE ATRIBUTES
C       (kx, ky, kz, iv, energy, i_region)    
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine scatter_carrier(n_lev)

       common
     &/ran_var/iso
     &/scatt_par/emax,de,w(10,5),tau_max(5),max_scatt_mech(5)
     &/scatt_par2/flag_mech(10,5)
     &/table/scatt_table(4000,10,5)
     &/particle_atr/kx,ky,kz,iv,e,i_region
      
      integer n_lev,iv,i_region
      real kx,ky,kz,e

C    Calculate index to the scattering table

      loc = int(e/de)
      if(loc.eq.0)loc=1
      if(loc.gt.n_lev)loc=n_lev
	   
C    Select scattering mechanism

      i_top = max_scatt_mech(i_region)
      rr = ran(iso)
      if(rr.ge.scatt_table(loc,i_top,i_region))then
c        freq(i_top+1,i_region)=freq(i_top+1,i_region)+1
         goto 222  ! self-scattering
      endif
      if(rr.lt.scatt_table(loc,1,i_region))then
         i_fix = 1
c        freq(i_fix,i_region)=freq(i_fix,i_region)+1
         goto 111
      endif
      if(i_top.gt.1)then
        do i=1,i_top-1
           bound_lower = scatt_table(loc,i,i_region)
           bound_upper = scatt_table(loc,i+1,i_region)
           if(rr.ge.bound_lower.and.rr.lt.bound_upper)then
              i_fix = i + 1
c             freq(i_fix,i_region)=freq(i_fix,i_region)+1
              goto 111
           endif
        enddo
      endif

111   continue

C    Perform scattering (change energy and randomize momentum)    

      select_mech = flag_mech(i_fix,i_region)
      if(select_mech.eq.1)then
         call isotropic_g(i_fix)
      elseif(select_mech.eq.2)then
         call isotropic_f(i_fix)
      elseif(select_mech.eq.3)then
         call Coulomb_angle_BH()
      endif

222   continue

      return
      end
      

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C       SCATTERING SUBROUTINES
C       (CHANGE ENERGY AND WAVEVECTORS OF PARTICLES)
C
C       In the definition of the scattering rates, a variable called
C       'flag_mech' has been defined.  The values assigned to this
C       variable correspond to:
C       
C       1  ==>  Isotropic scattering (acoustic, intervalley g-phonons)
C       2  ==>  Isotropic scattering (intervalley f-phonons)
C       3  ==>  Coulomb scattering ==> small-angle scattering
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C       ISOTROPIC SCATTERING PROCESS
C       uniform probability density for scattering in all directions
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine isotropic_g(i_fix)
       common
     &/ran_var/iso
     &/pi/pi,two_pi 
     &/mass/amd,amc,smh,hhm,tm(3),hm(3)
     &/nonp/af,af2,af4
     &/scatt_par/emax,de,w(10,5),tau_max(5),max_scatt_mech(5)
     &/particle_atr/kx,ky,kz,iv,e,i_region
      
      integer iv,i_region
      real kx,ky,kz,e
      
C     Update carrier energy
      e = e + w(i_fix,i_region)

C     Update carrier wavevector
      rknew = smh*sqrt(e*(1.+af*e))
      fi = two_pi*ran(iso)
      ct = 1.-2.*ran(iso)
      st = sqrt(1.-ct*ct)
      kx = rknew*st*cos(fi)
      ky = rknew*st*sin(fi)
      kz = rknew*ct  

      return
      end

     
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C       ISOTROPIC SCATTERING PROCESS
C       uniform probability density for scattering in all directions
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine isotropic_f(i_fix)
       common
     &/ran_var/iso
     &/pi/pi,two_pi 
     &/mass/amd,amc,smh,hhm,tm(3),hm(3)
     &/nonp/af,af2,af4
     &/scatt_par/emax,de,w(10,5),tau_max(5),max_scatt_mech(5)
     &/particle_atr/kx,ky,kz,iv,e,i_region
      
      integer iv,i_region
      real kx,ky,kz,e
      
C     Update carrier energy
      e = e + w(i_fix,i_region)

C     Update carrier wavevector
      rknew = smh*sqrt(e*(1.+af*e))
      fi = two_pi*ran(iso)
      ct = 1.-2.*ran(iso)
      st = sqrt(1.-ct*ct)
      kx = rknew*st*cos(fi)
      ky = rknew*st*sin(fi)
      kz = rknew*ct
      
      rr = ran(iso)
      if(iv.eq.1)then
         if(rr.le.0.5)then
            iv0 = 2
         else
            iv0 = 3
         endif
      endif
      if(iv.eq.2)then
         if(rr.le.0.5)then
            iv0 = 1
         else
            iv0 = 3
         endif
      endif
      if(iv.eq.3)then
         if(rr.le.0.5)then
            iv0 = 1
         else
            iv0 = 2
         endif
      endif
      iv = iv0               

      return
      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C      COULOMB SCATTERING ANGLE
C      Randomize the polar angle according to the expressions
C      derived in the notes
C      The assumption is that the scattering process is elastic
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine  Coulomb_angle_BH()

       common
     &/ran_var/iso
     &/pi/pi,two_pi 
     &/mass/amd,amc,smh,hhm,tm(3),hm(3)
     &/nonp/af,af2,af4
     &/scatt_par/emax,de,w(10,5),tau_max(5),max_scatt_mech(5)
     &/coulomb/doping_density(5),Energy_debye(5)
     &/particle_atr/kx,ky,kz,iv,e,i_region
      
      integer i_region,iv 
      real kx,ky,kz,e
      real k,kxy
      real kxp,kyp,kzp
      
C     Update carrier energy
c     enew = e + w(i_fix,i_region)

C     Calculate the rotation angles
      kxy = sqrt(kx*kx+ky*ky)
      k = sqrt(kxy*kxy+kz*kz)
      cth0 = kz/k
      sth0 = kxy/k
      cfi0 = kx/kxy
      sfi0 = ky/kxy

C     Randomize momentum in the rotated coordinate system
      ge = e*(1.+af*e)
      rr = ran(iso)
      cth = 1. - 2*rr/(1.+4.*(1-rr)*ge/Energy_debye(i_region))
      sth = sqrt(1.-cth*cth)
      fi = two_pi*ran(iso)
      cfi = cos(fi)
      sfi = sin(fi)
      kxp = k*sth*cfi
      kyp = k*sth*sfi
      kzp = k*cth

C     Return back to the original coordinate system
      kx = kxp*cfi0*cth0-kyp*sfi0+kzp*cfi0*sth0
      ky = kxp*sfi0*cth0+kyp*cfi0+kzp*sfi0*sth0
      kz = -kxp*sth0+kzp*cth0

c     e = enew   

      return
      end


