CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C       SUBROUTINE THAT CREATES THE SCATTERING TABLE
C       flag_mech = 1 ==> isotropic scattering process
C       flag_mech = 2 ==> polar optical phonons
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine sc_table(n_lev)

       common
     &/pi/pi,two_pi 
     &/temp/tem,Vt
     &/fund_const/q,h,kb,am0,eps_0
     &/mass/amd,amc,smh,hhm,tm(3),hm(3)
     &/nonp/af,af2,af4
     &/scatt_par/emax,de,w(10,5),tau_max(5),max_scatt_mech(5)
     &/scatt_par2/flag_mech(10,5)
     &/table/scatt_table(4000,10,5)     
     &/dielec_func/eps_high
     &/density/density,sound_velocity
     &/select_acouctic/acoustic_scattering
     &/select_Coulomb/Coulomb_scattering
     &/select_intervalley_1/intervalley_zero_g
     &/select_intervalley_2/intervalley_zero_f
     &/select_intervalley_3/intervalley_first_g
     &/select_intervalley_4/intervalley_first_f
     &/sigma_acoustic/sigma_acoustic
     &/doping_regions/n_region
     &/coulomb/doping_density(5),Energy_debye(5)
     &/Def_pot_1/DefPot_zero_g
     &/Def_pot_2/DefPot_zero_f
     &/Def_pot_3/DefPot_first_g
     &/Def_pot_4/DefPot_first_f
     &/interval_phonons_1/phonon_zero_g
     &/interval_phonons_2/phonon_zero_f
     &/interval_phonons_3/phonon_first_g
     &/interval_phonons_4/phonon_first_f
     &/acoustic/sigma
     &/intervalley1/coupling_constant
     &/intervalley2/delta_fi,final_valleys,i_mech

      integer n_lev,n_region
      character*30 out_file_1, out_file_2
      real kb

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     CREATE THE SCATTERING TABLE FOR EACH REGION
C     Scattering mechanism:  - Acoustic phonons
C                            - Intervalley g-phonons
C                            - Intervalley f-phonons    
C                            - Coulomb scattering
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      do i_g = 1, n_region

      i_count = 0
      
C    Acoustic phonons scattering rate

      if(acoustic_scattering.eq.1)then
         sigma = sigma_acoustic
         out_file_1 = 'acoustic_rate'
         call acoustic_rate(i_count,i_g,n_lev,out_file_1)
      endif      
      
C    Intervalley scattering: zero-order interaction (g-process)

      if(intervalley_zero_g.eq.1)then
         w0 = phonon_zero_g
         coupling_constant = DefPot_zero_g
         delta_fi = 0.
         final_valleys = 1.
         i_mech = 1
         out_file_1 = 'intervalley_zero_g_ab'
         out_file_2 = 'intervalley_zero_g_em'     
         call intervalley(i_count,i_g,n_lev,w0,
     1                    out_file_1,out_file_2)
      endif

C    Intervalley scattering: zero-order interaction (f-process)

      if(intervalley_zero_f.eq.1)then
         w0 = phonon_zero_f
         coupling_constant = DefPot_zero_f
         delta_fi = 0.
         final_valleys = 4.
         i_mech = 2
         out_file_1 = 'intervalley_zero_f_ab'
         out_file_2 = 'intervalley_zero_f_em'     
         call intervalley(i_count,i_g,n_lev,w0,
     1                    out_file_1,out_file_2)
      endif

C    Intervalley scattering: first-order interaction (g-process)

      if(intervalley_first_g.eq.1)then
         w0 = phonon_first_g
         coupling_constant = DefPot_first_g
         delta_fi = 0.
         final_valleys = 1.
         i_mech = 1
         out_file_1 = 'intervalley_first_g_ab'
         out_file_2 = 'intervalley_first_g_em'     
         call intervalley_first_order(i_count,i_g,n_lev,w0,
     1                    out_file_1,out_file_2)
      endif

C    Intervalley scattering: first-order interaction (f-process)

      if(intervalley_first_f.eq.1)then
         w0 = phonon_first_f
         coupling_constant = DefPot_first_f
         delta_fi = 0.
         final_valleys = 4.
         i_mech = 2
         out_file_1 = 'intervalley_first_f_ab'
         out_file_2 = 'intervalley_first_f_em'     
         call intervalley_first_order(i_count,i_g,n_lev,w0,
     1                    out_file_1,out_file_2)
      endif

C    Coulomb scattering rate - Brooks-Herring approach

      if(Coulomb_scattering.eq.1)then
         out_file_1 = 'Coulomb_rate'
         call Coulomb_BH(i_count,i_g,n_lev,out_file_1)
      endif

C    Need to add impact ionization
                       
      call renormalize_table(i_g,n_lev,i_count)
      print*,'Total number of mechanism included = ',i_count
      print*,'  '
      
      enddo
        
      return
      end
